import UrlPattern from 'url-pattern';
export { UrlPattern };
export declare type HandlerFunction = (...args: any[]) => Promise<Response | any | void>;
export interface RouteOptions {
    method?: string;
    matchUrl?: boolean;
}
export interface Route {
    pattern: UrlPattern;
    handler: HandlerFunction;
    options: RouteOptions;
}
export interface RouteResult {
    params: any | null;
    handler: HandlerFunction;
    url: URL;
    method: string;
}
export interface HandlerContext extends RouteResult {
    event: FetchEvent;
    request: FetchEvent['request'];
}
export declare class Router {
    private routes;
    all(pattern: string | UrlPattern, handler: HandlerFunction, options?: RouteOptions): Router;
    get(pattern: string | UrlPattern, handler: HandlerFunction, options?: RouteOptions): Router;
    post(pattern: string | UrlPattern, handler: HandlerFunction, options?: RouteOptions): Router;
    put(pattern: string | UrlPattern, handler: HandlerFunction, options?: RouteOptions): Router;
    patch(pattern: string | UrlPattern, handler: HandlerFunction, options?: RouteOptions): Router;
    delete(pattern: string | UrlPattern, handler: HandlerFunction, options?: RouteOptions): Router;
    head(pattern: string | UrlPattern, handler: HandlerFunction, options?: RouteOptions): Router;
    options(pattern: string | UrlPattern, handler: HandlerFunction, options?: RouteOptions): Router;
    private addRoute;
    findRoute(url: URL | string, method: string): RouteResult | null;
    findRouteForRequest(request: FetchEvent['request']): RouteResult | null;
    handleRequest(event: FetchEvent): Promise<Response | any | void> | null;
    watch(event: FetchEvent): void;
    clear(): void;
}
